program test_gcrs_str

! Writes a minimal 3-D structured grid CGNS file + cell-center-based
! flow solution (with some rind cell values) to it.

! Copyright (c) 2024 Francesco Poli <invernomuto@paranoici.org>
!
! Permission is hereby granted, free of charge, to any person obtaining
! a copy of this software and associated documentation files (the
! "Software"), to deal in the Software without restriction, including
! without limitation the rights to use, copy, modify, merge, publish,
! distribute, sublicense, and/or sell copies of the Software, and to
! permit persons to whom the Software is furnished to do so, subject to
! the following conditions:
!
! The above copyright notice and this permission notice shall be
! included in all copies or substantial portions of the Software.
!
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
! EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
! MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
! IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
! CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
! TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
! SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


use cgns

implicit none

#ifdef NEEDEDMACRO
double precision,pointer :: x(:,:,:),y(:,:,:),z(:,:,:),rho(:,:,:)
! N.B.: this could be real,pointer

integer :: isize(3,3)
integer :: irinddata(6)
character(32) :: basename,zonename,solname

integer :: i,j,k,ni,nj,nk
integer :: icelldim,iphysdim
integer :: ier
integer :: index_file,index_base,index_coord,index_zone,index_flow,index_field
#endif

! create minimal grid
ni = 3
nj = 4
nk = 2
allocate(x(ni,nj,nk),y(ni,nj,nk),z(ni,nj,nk))

do k = 1, nk
  do j = 1, nj
    do i = 1, ni
      x(i,j,k) = i-1
      y(i,j,k) = j-1
      z(i,j,k) = k-1
    end do
  end do
end do

! create minimal solution
allocate(rho(0:ni,0:nj,nk-1))

do k = 1, nk-1
  do j = 0, nj
    do i = 0, ni
      rho(i,j,k) = 1.0
      if ((i.eq.0).or.(i.eq.ni)) rho(i,j,k) = 0.0
      if ((j.eq.0).or.(j.eq.nj)) rho(i,j,k) = 0.0
    end do
  end do
end do

! open CGNS file for write
call cg_open_f('test.cgns',CG_MODE_WRITE,index_file,ier)
if (ier.ne.CG_OK) call cg_error_exit_f

! create base
basename = 'testBase'
icelldim = 3   ! volume cells
iphysdim = 3   ! in 3D space (3 coordinates are needed: x,y,z)
call cg_base_write_f(index_file, &
                     basename,icelldim,iphysdim, &
                     index_base,ier)

! define zone name
zonename = 'Zone1'

! vertex size
isize(1,1) = ni
isize(2,1) = nj
isize(3,1) = nk
! cell size (always 1 less than vertex size for structured grids)
isize(1,2) = isize(1,1) - 1
isize(2,2) = isize(2,1) - 1
isize(3,2) = isize(3,1) - 1
! boundary vertex size (always zero for structured grids)
isize(1,3) = 0
isize(2,3) = 0
isize(3,3) = 0
! create zone
call cg_zone_write_f(index_file,index_base, &
                     zonename,isize,Structured, &
                     index_zone,ier)

! write grid coordinates
! if x,y,z were real, the correct keyword would be RealSingle
call cg_coord_write_f(index_file,index_base,index_zone, &
                      RealDouble,'CoordinateX',x, &
                      index_coord,ier)
call cg_coord_write_f(index_file,index_base,index_zone, &
                      RealDouble,'CoordinateY',y, &
                      index_coord,ier)
call cg_coord_write_f(index_file,index_base,index_zone, &
                      RealDouble,'CoordinateZ',z, &
                      index_coord,ier)

! create solution
solname = 'Solution'

! create flow solution node
call cg_sol_write_f(index_file,index_base,index_zone, &
                    solname,CellCenter, &
                    index_flow,ier)

! go to position within tree at FlowSolution_t node
call cg_goto_f(index_file,index_base,ier,'Zone_t',index_zone, &
               'FlowSolution_t',index_flow,'end')

! write rind information under FlowSolution_t node
irinddata(1) = 1   ! i_lo
irinddata(2) = 1   ! i_hi
irinddata(3) = 1   ! j_lo
irinddata(4) = 1   ! j_hi
irinddata(5) = 0   ! k_lo
irinddata(6) = 0   ! k_hi
call cg_rind_write_f(irinddata,ier)

! write flow solution
! if rho were real, the correct keyword would be RealSingle
call cg_field_write_f(index_file,index_base,index_zone,index_flow, &
                      RealDouble,'Density',rho, &
                      index_field,ier)

! close CGNS file
call cg_close_f(index_file,ier)

deallocate(x,y,z,rho)

write(*,'(a)') 'Successfully wrote file test.cgns'

end
